//
//	AbsoluteLayout.java
//
//	version  1.0  98/01/15
//	platform Java 1.0
//	author   Mitsumasa IWATSUKI
//

package layout;

import java.awt.*;
import java.util.Hashtable;

public class AbsoluteLayout implements LayoutManager {
	protected Hashtable compMap;
	protected int width;
	protected int height;

	public AbsoluteLayout() {
		construct(0, 0);
	}

	public AbsoluteLayout(int width, int height) {
		construct(width, height);
	}

	public AbsoluteLayout(Dimension size) {
		construct(size.width, size.height);
	}

	public void setSize(int width, int height) {
		setSizeImpl(width, height);
	}

	public void setSize(Dimension size) {
		setSizeImpl(size.width, size.height);
	}

	public Dimension getSize() {
		return new Dimension(width, height);
	}

	public void addLayoutComponent(String name, Component comp) {
	}

	public void addLayoutComponent(Component comp, Object constraints) {
		if (constraints instanceof Rectangle) {
			Rectangle rect = (Rectangle)constraints;
			compMap.put(comp, new Rectangle(rect.x, rect.y, rect.width, rect.height));
		} else if (constraints != null) {
			throw new IllegalArgumentException("cannot add to layout: constraint must be a Rectangle");
		}
	}

	public void removeLayoutComponent(Component comp) {
		compMap.remove(comp);
	}

	public Dimension preferredLayoutSize(Container target) {
		return getLayoutSizeImpl(target);
	}

	public Dimension minimumLayoutSize(Container target) {
		return getLayoutSizeImpl(target);
	}

	public void layoutContainer(Container target) {
		Insets insets = target.insets();

		int count = target.countComponents();
		for (int i = 0; i < count; i++) {
			Component comp = target.getComponent(i);
			Rectangle rect = (Rectangle)compMap.get(comp);
			if (rect != null)
				comp.reshape(
					rect.x + insets.left, rect.y + insets.top,
					rect.width, rect.height);
		}
	}

	public String toString() {
		return getClass().getName() + "[width=" + width + ",height=" + height + "]";
	}

	// ---- helper methods ----

	protected void construct(int width, int height) {
		compMap = new Hashtable();
		setSizeImpl(width, height);
	}

	protected void setSizeImpl(int width, int height) {
		this.width = width;
		this.height = height;
	}

	protected Dimension getLayoutSizeImpl(Container target) {
		Insets insets = target.insets();
		return new Dimension(width + insets.left + insets.right, height + insets.top + insets.bottom);
	}
}
