//
//	FontBaseAbsoluteLayout.java
//
//	version  1.0  98/01/15
//	platform Java 1.0
//	author   Mitsumasa IWATSUKI
//

package layout;

import java.awt.*;
import java.util.Hashtable;

public class FontBaseAbsoluteLayout implements LayoutManager {
	protected Hashtable compMap;
	protected int width;
	protected int height;
	protected String baseString;
	protected static final String defaultBaseString = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";

	public FontBaseAbsoluteLayout() {
		construct(0, 0, defaultBaseString);
	}

	public FontBaseAbsoluteLayout(int width, int height) {
		construct(width, height, defaultBaseString);
	}

	public FontBaseAbsoluteLayout(Dimension size) {
		construct(size.width, size.height, defaultBaseString);
	}

	public FontBaseAbsoluteLayout(String baseString) {
		construct(0, 0, baseString);
	}

	public FontBaseAbsoluteLayout(int width, int height, String baseString) {
		construct(width, height, baseString);
	}

	public FontBaseAbsoluteLayout(Dimension size, String baseString) {
		construct(size.width, size.height, baseString);
	}

	public void setSize(int width, int height) {
		setSizeImpl(width, height);
	}

	public void setSize(Dimension size) {
		setSizeImpl(size.width, size.height);
	}

	public Dimension getSize() {
		return new Dimension(width, height);
	}

	public void setBaseString(String baseString) {
		setBaseStringImpl(baseString);
	}

	public String getBaseString() {
		return baseString;
	}

	public void addLayoutComponent(String name, Component comp) {
	}

	public void addLayoutComponent(Component comp, Object constraints) {
		if (constraints instanceof Rectangle) {
			Rectangle rect = (Rectangle)constraints;
			compMap.put(comp, new Rectangle(rect.x, rect.y, rect.width, rect.height));
		} else if (constraints != null) {
			throw new IllegalArgumentException("cannot add to layout: constraint must be a Rectangle");
		}
	}

	public void removeLayoutComponent(Component comp) {
		compMap.remove(comp);
	}

	public Dimension preferredLayoutSize(Container target) {
		return getLayoutSizeImpl(target);
	}

	public Dimension minimumLayoutSize(Container target) {
		return getLayoutSizeImpl(target);
	}

	public void layoutContainer(Container target) {
		Dimension fontSize = getFontSize(target);
		Insets insets = target.insets();

		int count = target.countComponents();
		for (int i = 0; i < count; i++) {
			Component comp = target.getComponent(i);
			Rectangle rect = (Rectangle)compMap.get(comp);
			if (rect != null) {
				int x = rect.x * fontSize.width / 4;
				int y = rect.y * fontSize.height / 8;
				int width = (rect.x + rect.width) * fontSize.width / 4 - x;
				int height = (rect.y + rect.height) * fontSize.height / 8 - y;
				comp.reshape(x + insets.left, y + insets.top, width, height);
			}
		}
	}

	public String toString() {
		return getClass().getName() + "[width=" + width + "(fbu),height=" + height + "(fbu),baseString='" + baseString +"']";
	}

	// ---- helper methods ----

	protected void construct(int width, int height, String baseString) {
		compMap = new Hashtable();
		setSizeImpl(width, height);
		setBaseStringImpl(baseString);
	}

	protected void setSizeImpl(int width, int height) {
		this.width = width;
		this.height = height;
	}

	protected void setBaseStringImpl(String baseString) {
		this.baseString = (baseString == null || baseString.length() == 0) ? defaultBaseString : baseString;
	}

	protected Dimension getLayoutSizeImpl(Container target) {
		Dimension fontSize = getFontSize(target);
		Insets insets = target.insets();
		return new Dimension(
			width * fontSize.width / 4 + insets.left + insets.right,
			height * fontSize.height / 8 + insets.top + insets.bottom);
	}

	protected Dimension getFontSize(Container target) {
		FontMetrics m = target.getFontMetrics(target.getFont());
		int width = m.stringWidth(baseString) / baseString.length();
		if (width <= 0)
			width = 1;
		int height = m.getHeight();
		return new Dimension(width, height);
	}
}
